package com.github.andrewlalis.running_every_day.view;

import com.github.andrewlalis.running_every_day.data.DataSource;

import javax.swing.*;
import java.awt.*;

/**
 * A panel for displaying a table view of run records, and various controls for
 * navigating the data.
 */
public class RunRecordsPanel extends JPanel {
    private final RunRecordTableModel tableModel;
    private final JTextField currentPageField;
    private final JButton firstPageButton;
    private final JButton previousPageButton;
    private final JButton nextPageButton;
    private final JButton lastPageButton;

    public RunRecordsPanel(DataSource dataSource) {
        super(new BorderLayout());
        this.tableModel = new RunRecordTableModel(dataSource);

        var table = new JTable(tableModel);
        table.getTableHeader().setReorderingAllowed(false);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
        table.getColumnModel().getColumn(0).setMaxWidth(40);
        table.getColumnModel().getColumn(1).setMaxWidth(80);
        table.getColumnModel().getColumn(2).setMaxWidth(80);
        table.getColumnModel().getColumn(3).setMaxWidth(100);
        table.getColumnModel().getColumn(4).setMaxWidth(80);
        table.getColumnModel().getColumn(5).setMaxWidth(80);
        var scrollPane = new JScrollPane(table, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        this.add(scrollPane, BorderLayout.CENTER);

        JPanel paginationPanel = new JPanel(new GridLayout(1, 6));
        firstPageButton = new JButton("First Page");
        firstPageButton.addActionListener(e -> {
            tableModel.firstPage();
            updateButtonStates();
        });
        previousPageButton = new JButton("Previous Page");
        previousPageButton.addActionListener(e -> {
            tableModel.previousPage();
            updateButtonStates();
        });
        nextPageButton = new JButton("Next Page");
        nextPageButton.addActionListener(e -> {
            tableModel.nextPage();
            updateButtonStates();
        });
        lastPageButton = new JButton("Last Page");
        lastPageButton.addActionListener(e -> {
            tableModel.lastPage();
            updateButtonStates();
        });

        JPanel currentPagePanel = new JPanel();
        currentPagePanel.add(new JLabel("Current Page: "));
        this.currentPageField = new JTextField("1", 3);
        currentPagePanel.add(this.currentPageField);

        JPanel pageSizePanel = new JPanel();
        pageSizePanel.add(new JLabel("Page Size: "));
        JComboBox<Integer> pageSizeSelector = new JComboBox<>(new Integer[]{5, 10, 20, 50, 100, 500});
        pageSizeSelector.setSelectedItem(tableModel.getPagination().size());
        pageSizeSelector.addItemListener(e -> {
            int size = (int) e.getItem();
            tableModel.setPageSize(size);
            updateButtonStates();
        });
        pageSizePanel.add(pageSizeSelector);

        paginationPanel.add(firstPageButton);
        paginationPanel.add(previousPageButton);
        paginationPanel.add(currentPagePanel);
        paginationPanel.add(pageSizePanel);
        paginationPanel.add(nextPageButton);
        paginationPanel.add(lastPageButton);

        this.add(paginationPanel, BorderLayout.SOUTH);
        SwingUtilities.invokeLater(() -> {
            tableModel.firstPage();
            updateButtonStates();
        });

        JPanel actionsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        JButton addActionButton = new JButton("Add a Record");
        actionsPanel.add(addActionButton);
        this.add(actionsPanel, BorderLayout.NORTH);
    }

    private void updateButtonStates() {
        firstPageButton.setEnabled(tableModel.canGoToFirstPage());
        nextPageButton.setEnabled(tableModel.canGoToNextPage());
        previousPageButton.setEnabled(tableModel.canGoToPreviousPage());
        lastPageButton.setEnabled(tableModel.canGoToLastPage());
        currentPageField.setText(String.valueOf(tableModel.getPagination().page() + 1));
    }
}
